#ifndef rest_intMATRIX_H
#define rest_intMATRIX_H
// ================ Include-Files ==== ==============================
#include "pch.h"
// =============== Klassen-Deklaration: intMATRIX  =======================================
// intMATRIX-Klasse fuer Matrizenoperationen vom Typ int
// (kann durch Template mit Klasse intMATRIX vereinigt werden)
// Rechteckmatrix mat(i,j,k,l) wird spaltenweise gespeichert !!!
// Indizierung von i,j,k,l beginnt bei 0 !!!
// vgl. U. Breymann: C++. Carl Hanser Verlag (1993), S.186 ff
// ======================================================================================
// Copyright: Univ.-Prof. Dr.-Ing. Udo F. Meissner, Erkrath / Darmstadt
// experimentelles Lehr- und Studienprogramm - kommerzielle Nutzung nicht gestattet
// Der Autor bernimmt keine Haftung fr Unvollstndigkeiten und Maengel der Algorithmen!
// Version:  1997
// Compiler: C++
// ========= Deklaration der Prototypem (header file) ===============
   class intMATRIX
{
// ==================================================================
//                 oeffentliche Mitgliedsfunktionen
//                 ================================
// 
//    ****** dieser Methodenteil ist im Buch dokumentiert ******* 
// 
// ==================================================================
//                 Geschuetzte Mitgliedsfunktionen
//                 ===============================
   protected:
// ==========
//
// Adresse von Matrixelement -> mat.adresse(i,j,k,l)
//==================================================
int *adresse(int i=0,int j=0,int k=0,int l=0);
//
// Matrixelement -> mat.matrix_element(i,j,k,l) vergleiche mat(i,j,k,l)
// ====================================================================
int &matrix_element(int i=0, int j=0, int k=0, int l=0);
//
// Matrix erzeugen -> erzeugen(ni,nj,nk,nl,neutyp,C_array)
// =======================================================
// bei neutyp=-1 temporaere Matrix erzeugen
// bei neutyp= 1 permanente Matrix erzeugen
// C_array vorinitialisiertes C_Feld [] = { ... }
// ---------------------------------------------------------------------------
void erzeugen(int ni=1,int nj=1,int nk=1,int nl=1,int neutyp=1,int* C_array=NULL);
//
// Matrix kopieren -> kopieren(&mat,neutyp)
// ========================================
// bei neutyp=-1 Matrix-Deskriptor, neuen Speicher anfordern (nur privat !!!)
// bei neutyp= 0 Matrix-Deskriptor, alten Speicher benutzen  (nur privat !!!)
// bei neutyp= 1 Matrix-Deskriptor, neuen Speicher anfordern, Matrix kopieren
// ---------------------------------------------------------------------------
void kopieren(intMATRIX &mat,int neutyp=1);
//
// temporaere Matrix erzeugen
//                          -> temporaere_matrix_erzeugen(ni,nj,nk,nl,neutyp)
// ===========================================================================
// ni, nj, nk, nl: Dimensionen der 4D-Matrix
// bei neutyp=-1 temporaere Matrix erzeugen
// bei neutyp= 1 permanente Matrix erzeugen
// -----------------------------------------
intMATRIX *temporaere_matrix_erzeugen(int ni,int nj=1,int nk=1,
                                                    int nl=1,int neutyp=-1);
//
// temporaere Matrix als Kopie erzeugen
//                                  -> temporaere_matrix_erzeugen(&mat,neutyp)
// ===========================================================================
// Kopie der Matrix mat:
// bei neutyp=-1 Matrix-Deskriptor, neuen Speicher anfordern (nur privat !!!)
// bei neutyp= 0 Matrix-Deskriptor, alten Speicher benutzen  (nur privat !!!)
// bei neutyp= 1 Matrix-Deskriptor, neuen Speicher anfordern, Matrix kopieren
// ---------------------------------------------------------------------------
intMATRIX *temporaere_matrix_erzeugen(intMATRIX &mat,int neutyp=-1);
//
// Matrix-Dimensionen temporaer aendern -> mat.dimension_temporaer(i,j,k,l)
// ===========================================================================
// Das Zusammenfassen von Indizes (Umdimensionierung) wird eigentlich nur
//                    fuer die Matrizenmultiplikation mat1*mat2 benoetigt.
// Daher gilt die Umdimensionierung nur bis zur naechsten Matrizenoperartion !
// Danach werden die Originaldimensionen wiederhergestellt !!!
// ---------------------------------------------------------------------------
void dimension_temporaer(int ni=1,int nj=1,int nk=1,int nl=1);
//
// Teilmatrix heraus -> ...=mat1.teil_heraus(mat2, i,j,k,l)
// =========================================================
// teil_heraus holt ab Index mat2(i,j,k,l) aufeinanderfolgende Matrixelemente
// heraus, die in der Teilmatrix mat1(ni,nj,nk,nl) gespeichert werden
// ---------------------------------------------------------------------------
void teil_heraus(intMATRIX &mat2, int i=0,int j=0,int k=0,int l=0);
//
// Teilmatrix hinein -> mat.teil_hinein(i,j,k,l)=...
// =================================================
// teil_hinein erlaubt das Einspeichern der Teilmatrix ab Index mat(i,j,k,l)
// ---------------------------------------------------------------------------
void teil_hinein(int i=0, int j=0, int k=0, int l=0);
//
// Untermatrix heraus -> ...=mat1.unter_heraus(mat2, i,j,k,l)
// ==========================================================
// unter_heraus holt ab Index mat2(i,j,k,l)
//                                    die Untermatrix mat1(ni,nj,nk,nl) heraus
// ---------------------------------------------------------------------------
void unter_heraus(intMATRIX &mat1, int i=0,int j=0,int k=0,int l=0);
//
// Untermatrix hinein -> mat.unter_hinein(i,j,k,l)=...
// ===================================================
// unter_hinein erlaubt das Einspeichern der Untermatrix ab Index mat(i,j,k,l)
// ---------------------------------------------------------------------------
void unter_hinein(int i=0,int j=0,int k=0,int l=0);
//
// Matrix-Ausgabe -> mat.ausgeben(s)
// =================================
void ausgeben(ostream &s);
//
// Matrix-Eingabe -> mat.einlesen(s)
// =================================
void einlesen(istream &s);
//
// Alle Matrix-Elemente belegen -> mat.zuweisen(zahl,zweig)
//                                 (nur Matrizeninhalt kopieren)
// =============================================================
// Zweig= 0 -> mat=    zahl
//      = 2 -> mat=mat+zahl
//      = 4 -> mat=mat-zahl
// ------------------------
void zuweisen(int zahl,int zweig=0);
//
// Matrix zuweisen -> mat1.zuweisen(mat2,zweig)
//                    (nur Matrizeninhalt kopieren)
// ================================================
// Zweig= 0 -> mat1=    mat2
//      = 2 -> mat1=mat+mat2
//      = 4 -> mat1=mat-mat2
void zuweisen(intMATRIX &mat2,int zweig=0);
//
// Addition -> mat1+mat2 bzw. mat1+vek2 bzw. vek1+mat2
// ===================================================
void addieren(intMATRIX &mat1,intMATRIX &mat2);
//
// Subtraktion -> mat1-mat2 bzw. mat1-vek2 bzw. vek1-mat2
// ======================================================
void subtrahieren(intMATRIX &mat1,intMATRIX &mat2);
//
// ======================================================================
//                 Private Mitgliedsfunktionen
//                 ===========================
   private:
// ========
//
// Konstruktor -> intMATRIX  mat(ni,nj,nk,nl,neutyp)
// ========================================================
// ni, nj, nk, nl: Dimensionen der 4D-Matrix
// bei neutyp=-1 temporaere Matrix erzeugen
// bei neutyp= 1 permanente Matrix erzeugen 
// ---------------------------------------------------------------------------
intMATRIX (int ni,int nj,int nk,int nl,int neutyp);
//
// Kopier-Konstruktor -> intMATRIX  mat1(&mat2,neutyp)
// ===================================================
// bei neutyp=-1 Matrix-Deskriptor, neuen Speicher anfordern (nur privat !!!)
// bei neutyp= 0 Matrix-Deskriptor, alten Speicher benutzen  (nur privat !!!)
// bei neutyp= 1 Matrix-Deskriptor, neuen Speicher anfordern, Matrix kopieren
// ---------------------------------------------------------------------------
intMATRIX (intMATRIX  &mat2,int neutyp);
//
// Index ijkl von Matrixelement -> mat.index(i,j,k,l)
// ==================================================
int index(int i=0, int j=0, int k=0, int l=0);
//
// 2D-Indizes von Matrizenelement -> mat.index_2d(ij, dimi,dimj, i,j)
// Eingabe: 1D-Index ij, Dimensionen dimi,dimj, Ausgabe: 2D-Indizes i,j
// ====================================================================
void index_2d(int ij,int dimi,int dimj,int &i,int &j);
//
// 3D-Indizes von Matrizenelement -> mat.index_3d(ijk, dimi,dimj,dimk, i,j,k)
// Eingabe: 1D-Index ijk,Dimensionen dimi,dimj,dimk, Ausgabe: 3D-Indizes i,j,k
// ===========================================================================
void index_3d(int ijk,int dimi,int dimj,int dimk,int &i,int &j,int &k);
//
// 4D-Indizes von Matrizenelement -> mat.index_4d(ijkl, i,j,k,l)
// Eingabe: 1D-Index ijkl, Ausgabe: 4D-Indizes i,j,k,l
// =============================================================
void index_4d(int ijkl,int &i,int &j,int &k,int &l);
// ======================================================================
//                 Geschuetzte Datenkomponenten -> Matrix-Deskriptor
//                 =================================================
   protected:
// ==========
//
// Typ der Matrix ->  2 = permanent, gegen Loeschen gesperrt
// -------------- ->  1 = permanent
//                ->  0 = temporaer mit altem Speicher
//                -> -1 = temporaer mit neuem Speicher
int typ;
//
// Dimensionen der 4D-Matrix mat(dimi,dimj,dimk,diml)
// -----------
int dimi,dimj,dimk,diml;
//
// Gesamtdimension dimmax=dimi*dimj*dimk*diml des Vektors matrix[dimmax]
// ---------------
int dimmax;
//
// Vektor matrix[dmax] zur Speicherung der Matrix mat(di,dj,dk,dl)
int *matrix;
//
// Zeiger auf aktuelles Matrix-Element fr die Inkrement-Operatoren
//                                           &operator ++ (), &operator -- () 
int *aktuelles_element;
// ===========================================================================
//                 Private Datenkomponenten -> Matrix-Deskriptor
//                 =============================================
   private:
// ========
//
// Anfangsindizes
// --------------
int ia,ja,ka,la;
//
// Typ des Einspeichern fuer den Zuweisungsoperator &operator = ()
// --------------------
//                ->  1 = Matrix
//                ->  0 = Teil-Matrix  -> Funktion  teil_hinein(ia,ja,ka,la)
//                -> -1 = Unter-Matrix -> Funktion unter_hinein(ia,ja,ka,la)
int eintyp;
};
// ============================================================================
#endif